function [xvals,fvals,xhist,fhist] = bracket(costf,x0)
% function [xvals,fvals,xhist,fhist] = bracket(costf,x0)
% find a bracket for a minimum of the function costf
% on the interval [0,infinity)
% We assume that the function is decreasing at x=0 !!!!!!
% if you have a good guess, give it as x0,
% otherwise set x0 to the empty matrix: x0 = []
% and this function will use a default of x0=1

if( isempty(x0) )
  x0 = 1;
end

% First, we'll set up the history arrays:
fhist = [];
xhist = [];

% We'll find a bracket of the form a < b < c
aval = 0;
bval = x0;
afval = costf(aval);
bfval = costf(bval);

xhist(1) = aval;
fhist(1) = afval;
% the next line uses the "end" symbol to mean "last element of the array".
% in Scilab, this is done with the symbol "$" instead of "end".
xhist(end+1) = bval;
fhist(end+1) = bfval;

if( afval < bfval )
  % cut the bracket in half, etc.
  while( afval < bfval)
    cval = bval;
    cfval = bfval;
    bval = bval/2;
    bfval = costf(bval);
    xhist(end+1) = bval;
    fhist(end+1) = bfval;
  end % while
else % afval > bfval, need to expand bracket
  cval = bval*2;
  cfval = costf(cval);
  xhist(end+1) = cval;
  fhist(end+1) = cfval;

  while( bfval > cfval )
    aval = bval;
    afval = bfval;
    bval=cval;
    bfval = cfval;
    cval=cval*2;
    cfval = costf(cval);
    xhist(end+1) = cval;
    fhist(end+1) = cfval;
  end %while
end % if

% at this point, our bracket is aval,bval,cval
xvals = [aval, cval];
fvals = [afval, cfval];


% Try this to run it:
% f=@(x) (x-10).^2;
% [xvals,fvals,xhist,fhist] = bracket(f,1)

% Qmat = [1 0; 0 3];
% f=@(x) x' * Qmat * x;
% [xvals,fvals,xhist,fhist] = bracket(f,1)
