function [f,xopt,gradopt,xhist,fhist,ahist] = steepest_descent(sdcostf,x0,params)
% function [f,xopt,gradopt,xhist,fhist,ahist] = steepest_descent(costf,x0,params)
% inputs and outputs work mostly like fminunc()
% ahist is an "action history": it says whether each point in xhist
% was a steepest-descent point ('M' for Major iteration),
% or a line-search point ('L' for line search).


[ofv, grd, hes] = sdcostf(x0,params);

xhist=x0(:); % xhist will have one _column_ for each x vector;
% it will grow toward the right, rather than downward.
fhist(1)=ofv;
ahist(1)='M';

xvec = x0;
old_ofv = ofv ;
not_there_yet = 1;  % just to get into the while loop
while( not_there_yet )
  dvec = -grd;
  linesearchhelper=@(alpha) sdcostf(xvec+alpha*dvec,params);

  
  starting_alpha = 1; % we don't have any better ideas, so start with 1.0
  % xhistL and fhistL: the L refers to Line search.
  
  [ofv,alpha_opt,xhistL,fhistL] = goldensection(linesearchhelper,starting_alpha);
  % actually, I don't feel like adding the line search history into our history.
  
  % take the step according to the alpha we found:
  xvec = xvec + alpha_opt * dvec;
  xhist = [xhist, xvec(:)];
  fhist = [fhist, ofv];
  ahist = [ahist, 'M'];
  % we got ofv from the line search, but we also want the gradient.
%  disp('calling sdcostf')
  [ofv, grd, hes] = sdcostf(xvec,params);
  % the really chintzy way to decide if we're done yet: 100 major iterations?
  not_there_yet = sum(ahist=='M') <= 100;
  % Let's stop if we see less than a 1 _percent_ change in obj.func.value
%  not_there_yet = (ofv-old_ofv)/old_ofv < 1 / 100.0 ; 
 old_ofv = ofv;
end % while

% finalize the return values
f = ofv;
xopt = xvec;
gradopt = grd;


% Qmat = [1 0 ; 0 3];
% x0 = [1;1];
% [f,xopt,gradopt,xhist,fhist] = steepest_descent(@purequad,x0,Qmat)
% plot(xhist(1,:),xhist(2,:),'.r-')

% Here are the ways Matlab decides to stop:
%      1  Magnitude of gradient smaller than the specified tolerance. 
%      2  Change in X smaller than the specified tolerance.
%      3  Change in the objective function value smaller than the specified 
%          tolerance (only occurs in the large-scale method).
%      0  Maximum number of function evaluations or iterations reached.
%     -1  Algorithm terminated by the output function.
%     -2  Line search cannot find an acceptable point along the current
%          search direction (only occurs in the medium-scale method).